import { useState, useEffect} from "react";

const types = {
  email: {
    regex: /^(([^<>()[\]\\.,;:\s@"]+(\.[^<>()[\]\\.,;:\s@"]+)*)|(".+"))@((\[[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\])|(([a-zA-Z\-0-9]+\.)+[a-zA-Z]{2,}))$/,
    message: "Preencha um e-mail válido",
  },
  password: {
    regex: /^(?=.*[a-z])(?=.*[A-Z])(?=.*[0-9])(?=.*[!@#%&])/,
    message:
      "A senha precisa ter 1 caracter maíusculo, 1 minúsculo e 1 digito. Com no mínimo 8 caracteres.",
  },
  number: {
    regex: /^\d+$/,
    message: "Utilize números apenas.",
  },
};

export default function useForm(validation) {
  const [value, setValue] = useState('');
  const [error, setError] = useState('');

  function handleValidate(value) {
    setError('');
    if (validation.required && !value) {
      setError(validation.messages?.required || "Preencha um valor.");
    }
    if (validation.maxLength && value && value.length > validation.maxLength) {
      setError(validation.messages?.maxLength.replace('$[]', validation?.maxLength) || `Máximo de ${validation.maxLength} caracteres permitidos`)
    }
    if (validation.minLength && value && value.length < validation.minLength) {
      setError(validation.messages?.minLength.replace('$[]', validation?.minLength) || `Mínimo de ${validation.minLength} caracteres exigidos`);
    }
    if (value && validation.email && !types['email'].regex.test(value)) {
      setError(validation.messages?.email || `Insira um email válido`);
    }
    if (value && validation.number && !types['number'].regex.test(value)) {
      setError(validation.messages?.number || `Insira apenas números`);
    }
    if (value && validation.password && !types['password'].regex.test(value)) {
      setError(validation.messages?.password || `A senha precisa ter 1 caracter maíusculo, 1 minúsculo e 1 digito.`);
    }
  }

  function onChange(e) {
    if (error) {
        handleValidate((validation.trimValue) ? e.target.value.trim() : (e.target.value));
    }
    setValue((validation.trimValue) ? e.target.value.trim() : (e.target.value));
  }

  return {
    setValue,
    value,
    onChange,
    error,
    handleValidate: (value) => handleValidate(value),
    validate: () => handleValidate(value)
  };
}
